%bcond_without testsuite
%bcond_with doc
%bcond_with debuginfod

%global dyninst_base dyninst-%%{version}
%global testsuite_base testsuite-%%{version}

Summary:		Tools for binary instrumentation, analysis, and modification
Name:			dyninst
Version:		12.3.0
Release:		3%{?dist}
License:		LGPLv2+
URL:			http://www.dyninst.org
Source0:		https://github.com/dyninst/dyninst/archive/v%{version}/dyninst-%{version}.tar.gz
Source1:		https://github.com/dyninst/testsuite/archive/refs/tags/v%{version}.tar.gz #/testsuite-%{version}.tar.gz
Patch3000:		onetbb.patch

BuildRequires:	gcc-c++ cmake
BuildRequires:	elfutils-devel elfutils-libelf-devel
%if %{with debuginfod}
BuildRequires:	elfutils-debuginfod-client-devel
%endif
BuildRequires:	boost-devel binutils-devel
BuildRequires:	libtirpc-devel tbb tbb-devel
%if %{with doc}
BuildRequires:	tex-latex
%endif
%if %{with testsuite}
BuildRequires:	gcc-gfortran make
BuildRequires:	libxml2-devel
%{?filter_setup:
%filter_provides_in %{_libdir}/dyninst/testsuite/
%filter_requires_in %{_libdir}/dyninst/testsuite/
%filter_setup
}
%endif
ExclusiveArch:	x86_64 ppc64le aarch64

%description
Dyninst is an Application Program Interface (API) to permit the insertion of 
code into a computer application that is either running or on disk. The API 
for inserting code into a running application, called dynamic instrumentation, 
shares much of the same structure as the API for inserting code into an executable 
file or library, known as static instrumentation. The API also permits changing 
or removing subroutine calls from the application program. Binary code changes 
are useful to support a variety of applications including debugging, performance 
monitoring, and to support composing applications out of existing packages. The 
goal of this API is to provide a machine independent interface to permit the 
creation of tools and applications that use runtime and static code patching. 

%package devel
Summary:	Header files and libraries for dyninst development
Requires:	dyninst = %{version}-%{release}
Requires:	boost-devel tbb-devel

%description devel
This package provides Header files and libraries for dyninst development.

%if %{with doc}
%package doc
Summary:	Documentation for using the Dyninst API

%description doc
This package provides API documentation for the Dyninst libraries.
%endif

%if %{with testsuite}
%package testsuite
Summary:	The testsuite for Dyninst and Dyninst component libraries
Requires:	dyninst = %{version}-%{release}
Requires:	dyninst-devel = %{version}-%{release}

%description testsuite
This package provides the testsuite for Dyninst and Dyninst component libraries.
%endif

%prep
%setup -q -n %{name}-%{version} -c

%if %{with testsuite}
%setup -q -T -D -a 1
%endif

pushd %{dyninst_base}
%patch3000 -p1
popd

%build
cd %{dyninst_base}

CFLAGS="$CFLAGS $RPM_OPT_FLAGS"
LDFLAGS="$LDFLAGS $RPM_LD_FLAGS"
CXXFLAGS="$CFLAGS"
export CFLAGS CXXFLAGS LDFLAGS

%cmake \
%if %{with debuginfod}
 -DENABLE_DEBUGINFOD=1 \
%endif
 -DINSTALL_LIB_DIR:PATH=%{_libdir}/dyninst \
 -DINSTALL_INCLUDE_DIR:PATH=%{_includedir}/dyninst \
 -DINSTALL_CMAKE_DIR:PATH=%{_libdir}/cmake/Dyninst \
 -DCMAKE_BUILD_TYPE=None \
 -DCMAKE_SKIP_RPATH:BOOL=YES
%cmake_build

DESTDIR="../install" %__cmake --install "%{__cmake_builddir}"
find ../install -name '*.cmake' -execdir \
     sed -i -e "s!%{_prefix}!$PWD/../install&!" '{}' '+'
sed -i '/libtbb.so/ s/".*usr/"\/usr/' $PWD/../install%{_libdir}/cmake/Dyninst/commonTargets.cmake

%if %{with testsuite}
cd ../%{testsuite_base}
%cmake \
 -DDyninst_DIR:PATH=$PWD/../install%{_libdir}/cmake/Dyninst \
 -DINSTALL_DIR:PATH=%{_libdir}/dyninst/testsuite \
 -DCMAKE_BUILD_TYPE:STRING=Debug \
 -DCMAKE_SKIP_RPATH:BOOL=YES
%cmake_build
%endif

%install
cd %{dyninst_base}
%cmake_install

%if %{with testsuite}
  rm -f %{buildroot}%{_docdir}/*-%{version}.pdf
  cd ../%{testsuite_base}
  %cmake_install
  find %{buildroot}%{_libdir}/dyninst/testsuite/ \
    -type f '!' -name '*.a' -execdir chmod 644 '{}' '+'
%else
  rm -f %{buildroot}%{_bindir}/parseThat
%endif

mkdir -p %{buildroot}/etc/ld.so.conf.d
echo "%{_libdir}/dyninst" > %{buildroot}/etc/ld.so.conf.d/%{name}-%{_arch}.conf

%files
%license %{dyninst_base}/COPYRIGHT %{dyninst_base}/LICENSE.md
%config(noreplace) /etc/ld.so.conf.d/*
%dir %{_libdir}/dyninst
%{_libdir}/dyninst/*.so.*
%{_libdir}/dyninst/libdyninstAPI_RT.so
%{_libdir}/dyninst/libdyninstAPI_RT.a

%files devel
%{_includedir}/dyninst
%{_libdir}/dyninst/*.so
%{_libdir}/cmake/Dyninst

%if %{with doc}
%files doc
%doc %{dyninst_base}/dataflowAPI/doc/dataflowAPI.pdf
%doc %{dyninst_base}/dynC_API/doc/dynC_API.pdf
%doc %{dyninst_base}/dyninstAPI/doc/dyninstAPI.pdf
%doc %{dyninst_base}/instructionAPI/doc/instructionAPI.pdf
%doc %{dyninst_base}/parseAPI/doc/parseAPI.pdf
%doc %{dyninst_base}/patchAPI/doc/patchAPI.pdf
%doc %{dyninst_base}/proccontrol/doc/proccontrol.pdf
%doc %{dyninst_base}/stackwalk/doc/stackwalk.pdf
%doc %{dyninst_base}/symtabAPI/doc/symtabAPI.pdf
%endif

%if %{with testsuite}
%files testsuite
%{_bindir}/parseThat
%dir %{_libdir}/dyninst/testsuite/
%attr(755,root,root) %{_libdir}/dyninst/testsuite/*[!a]
%attr(644,root,root) %{_libdir}/dyninst/testsuite/*.a
%endif

%changelog
* Wed Jan 17 2024 Rebuild Robot <rebot@opencloudos.org> - 12.3.0-3
- Rebuilt for elfutils

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 12.3.0-2
- Rebuilt for OpenCloudOS Stream 23.09

* Wed Aug 02 2023 rockerzhu <rockerzhu@tencent.com> - 12.3.0-1
- Update 12.3.0 to build with boost 1.82.0.

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 12.2.0-4
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 12.2.0-3
- Rebuilt for OpenCloudOS Stream 23

* Thu Jan 12 2023 Xiaojie Chen <jackxjchen@tencent.com> - 12.2.0-2
- Fix bogus version in changelog

* Fri Sep 16 2022 Xiaojie Chen <jackxjchen@tencent.com> - 12.2.0-1
- Initial build
